// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PixelPerfectCamera.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PixelPerfectCamera::PixelPerfectCamera(std::weak_ptr<SceneItem> item) :
  SceneItemComponent(item),
  scale(1.0f),
  camera(nullptr)
{

}

void PixelPerfectCamera::updateDimensions() {
  if(this->camera == nullptr) return;

  float_t w, h;

  auto target = this->camera->getRenderTarget();
  switch(this->camera->type) {
    case CAMERA_TYPE_ORTHONOGRAPHIC:
      w = target->getWidth() / 2.0f / (float_t)this->scale;
      h = target->getHeight() / 2.0f / (float_t)this->scale;
      camera->orthoLeft = -w;
      camera->orthoRight = w;
      camera->orthoTop = h;
      camera->orthoBottom = -h;
      break;
    
    case CAMERA_TYPE_PERSPECTIVE:
      item.lock()->lookAtPixelPerfect(
        glm::vec3(0, 0, 0),
        glm::vec3(0, 0, 0),
        target->getHeight() / this->scale,
        this->camera->fov
      );
      // this->lookAt(glm::vec3(360, 360, 360), glm::vec3(0, 0, 0));
      break;
    
    default:
      assertUnreachable("PixelPerfectCamera::updateDimensions: Unknown camera type");
  }
}

void PixelPerfectCamera::onStart() {
  if(camera == nullptr) camera = item.lock()->getComponent<Camera>();
  
  useEffect([&]{
    this->updateDimensions();
  }, scale);

  useEffectWithTeardown([&]{
    if(!camera._realValue) {
      return teardown = [&]{};
    }

    this->updateDimensions();

    return teardown = [&]{
      useEvent([&](float_t w, float_t h){
        this->updateDimensions();
      }, this->camera->eventRenderTargetResized);
    };
  }, camera)();
}