// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnopengl.hpp"
#include "display/ITexture.hpp"

namespace Dawn {
  class TextureRenderTarget;
  
  typedef GLuint textureslot_t;

  class Texture : public ITexture {
    private:
      int32_t width = -1;
      int32_t height = -1;
      GLuint id = -1;
      enum TextureFormat format;
      enum TextureDataFormat dataFormat;

      void updateTextureProperties();
      void bufferRaw(const void *data);

    public:
      Texture();
      int32_t getWidth() override;
      int32_t getHeight() override;
      void setSize(
        const int32_t width,
        const int32_t height,
        const enum TextureFormat format,
        const enum TextureDataFormat dataFormat
      ) override;
      void fill(const struct Color) override;
      void fill(const uint8_t) override;
      bool_t isReady() override;
      void buffer(const struct ColorU8 pixels[]) override;
      void buffer(const struct Color pixels[]);
      void buffer(const uint8_t pixels[]) override;
      
      /**
       * Binds the texture to the given slot (for use by the shaders).
       * 
       * @param slot Slot to bind to.
       */
      void bind(const textureslot_t slot);

      ~Texture();

      friend class TextureRenderTarget;
  };
}